# -*- coding: utf-8 -*-

from Products.MeetingCommunes.tests.MeetingCommunesTestCase import MeetingCommunesTestCase


class testCustomMeeting(MeetingCommunesTestCase):
    """
        Tests the Meeting adapted methods
    """
    def test_InitializeDecisionField(self):
        """
            In the doDecide method, we initialize the Decision field to a default value made of
            Title+Description if the field is empty...
        """
        # make sure we are not hit by any other xhtmlTransformations
        self.meetingConfig.setXhtmlTransformTypes(())
        # check that it works
        # check that if the field contains something, it is not intialized again
        self.changeUser('pmManager')
        # create some items
        # empty decision
        i1 = self.create('MeetingItem', title='Item1', description="<p>Description Item1</p>")
        i1.setDecision("")
        i1.setProposingGroup(self.developers_uid)
        # decision field is already filled
        i2 = self.create('MeetingItem', title='Item2', description="<p>Description Item2</p>")
        i2.setDecision("<p>Decision Item2</p>")
        i2.setProposingGroup(self.developers_uid)
        # create an item with the default Kupu empty value
        i3 = self.create('MeetingItem', title='Item3', description="<p>Description Item3</p>")
        i3.setDecision("<p>&nbsp;</p>")
        i3.setProposingGroup(self.developers_uid)
        meeting = self.create('Meeting', date='2007/12/11 09:00:00')
        # present every items in the meeting
        items = (i1, i2, i3)
        for item in items:
            self.presentItem(item)
        # check the decision field of every item
        self.assertTrue(i1.getDecision(keepWithNext=False) == "")
        self.assertTrue(i2.getDecision(keepWithNext=False) == '<p>Decision Item2</p>')
        self.assertTrue(i3.getDecision(keepWithNext=False) == '<p>&nbsp;</p>')
        # if cfg.initItemDecisionIfEmptyOnDecide is False, the decision field is not initialized
        self.meetingConfig.setInitItemDecisionIfEmptyOnDecide(False)
        self.decideMeeting(meeting)
        self.assertTrue(i1.getDecision(keepWithNext=False) == "")
        self.assertTrue(i2.getDecision(keepWithNext=False), '<p>Decision Item2</p>')
        # a complex HTML is not 'touched'
        self.assertTrue(i3.getDecision(keepWithNext=False), '<p>&nbsp;</p>')
        # now if cfg.initItemDecisionIfEmptyOnDecide is True
        # fields will be initialized
        self.meetingConfig.setInitItemDecisionIfEmptyOnDecide(True)
        # decide the meeting again
        self.backToState(meeting, 'created')
        self.decideMeeting(meeting)
        # i1 should contains now the concatenation of title and description
        self.assertEqual(i1.getDecision(keepWithNext=False), '<p>Item1</p><p>Description Item1</p>')
        # i2 sould not have changed
        self.assertEqual(i2.getDecision(keepWithNext=False), '<p>Decision Item2</p>')
        # i3 is initlaized because the decision field contained an empty_value
        self.assertEqual(i3.getDecision(keepWithNext=False), '<p>Item3</p><p>Description Item3</p>')

    def test_GetNumberOfItems(self):
        """
          This method will return a certain number of items depending on passed paramaters.
        """
        self.changeUser('admin')
        # make categories available
        self.setMeetingConfig(self.meetingConfig2.getId())
        self.changeUser('pmManager')
        meeting = self._createMeetingWithItems()
        orderedItems = meeting.getItems(ordered=True)
        # the meeting is created with 5 items
        self.assertEqual(len(orderedItems), 5)
        itemUids = [item.UID() for item in orderedItems]
        # without parameters, every items are returned
        self.assertEqual(meeting.adapted().getNumberOfItems(itemUids), 5)

        # test the 'privacy' parameter
        # by default, 2 items are 'secret' and 3 are 'public'
        itemPrivacies = [item.getPrivacy() for item in orderedItems]
        self.assertEqual(itemPrivacies.count('secret'), 2)
        self.assertEqual(itemPrivacies.count('public'), 3)
        # same using getNumberOfItems
        self.assertEqual(meeting.adapted().getNumberOfItems(itemUids, privacy='secret'), 2)
        self.assertEqual(meeting.adapted().getNumberOfItems(itemUids, privacy='public'), 3)

        # test the 'categories' parameter
        # by default, 2 items are in the 'events' category,
        # 2 are in the 'development' category
        # 1 in the 'research' category
        itemCategories = [item.getCategory() for item in orderedItems]
        self.assertEqual(itemCategories.count('events'), 2)
        self.assertEqual(itemCategories.count('development'), 2)
        self.assertEqual(itemCategories.count('research'), 1)
        # same using getNumberOfItems
        self.assertEqual(meeting.adapted().getNumberOfItems(itemUids, categories=['events', ]), 2)
        self.assertEqual(meeting.adapted().getNumberOfItems(itemUids, categories=['development', ]), 2)
        # we can pass several categories
        self.assertEqual(
            meeting.adapted().getNumberOfItems(
                itemUids, categories=['dummycategory', 'research', 'development', ]),
            3)

        # test the 'late' parameter
        # by default, no items are late so make 2 late items
        # remove to items, freeze the meeting then add the items
        item1 = orderedItems[0]
        item2 = orderedItems[1]
        self.backToState(item1, 'proposed')
        self.backToState(item2, 'proposed')
        self.freezeMeeting(meeting)
        item1.setPreferredMeeting(meeting.UID())
        item2.setPreferredMeeting(meeting.UID())
        self.presentItem(item1)
        self.presentItem(item2)
        # now we have 4 normal items and 2 late items
        self.assertEqual(len(meeting.getItems()), 5)
        self.assertEqual(len(meeting.getItems(listTypes=['late'])), 2)
        # same using getNumberOfItems
        self.assertEqual(meeting.adapted().getNumberOfItems(itemUids, listTypes=['normal']), 3)
        self.assertEqual(meeting.adapted().getNumberOfItems(itemUids, listTypes=['late']), 2)

        # we can combinate parameters
        # we know that we have 2 late items that are using the 'development' category...
        lateItems = meeting.getItems(listTypes=['late'])
        self.assertEqual(len(lateItems), 2)
        self.assertEqual(lateItems[0].getCategory(), 'development')
        self.assertEqual(lateItems[1].getCategory(), 'development')
        self.assertEqual(
            meeting.adapted().getNumberOfItems(
                itemUids, categories=['development', ],
                listTypes=['late']),
            2)
        # we have so 0 normal item using the 'development' category
        self.assertEqual(
            meeting.adapted().getNumberOfItems(
                itemUids, categories=['development', ], listTypes=['normal']),
            0)
